#include <windows.h>
#include "Event.h"
#include "Constants.h"
#include "screen.h"
#include "object.h"


Event::Event()
{
}

Event::Event(void *entity, int eventtype, int extrainfo, Ticker *cl)
{
	victim = NULL;
	attacker = NULL;
	charact = NULL;
	ent = entity;
	type = eventtype;
	extra = extrainfo;
}

//used for EVENT_MOVEMENT and EVENT_STATIC so far.
Event::Event(struct cell *c, void *entity, int eventtype, int extrainfo, double displacementx, double displacementy)
{
	w = c;
	ent = entity;
	charact = NULL;
	victim = NULL;
	attacker = NULL;
	type = eventtype;
	extra = extrainfo;
	deltax = displacementx;
	deltay = displacementy;
}

//used for EVENT_MELEE and EVENT_HIT so far.
Event::Event(struct character *attack, struct character *target, int eventtype, float displacement, int damage)
{
	w = NULL;
	totaldamage = damage;
	charact = NULL;
	attacker = attack;
	victim = target;
	type = eventtype;
	deltax = displacement;
}

//used for EVENT_DEATH
Event::Event(struct cell *c, struct character *chrct, int eventtype, float displacement)
{
	w = c;
	charact = chrct;
	victim = NULL;
	attacker = NULL;
	type = eventtype;
	deltax = displacement;
}

Event::~Event()
{
	//when the Event finishes reset framebegin and frameend in char
	// IF it was a char
	if(type == EVENT_MOVEMENT){
		charact->framebegin = 0.0;
		charact->frameend = 1.0 / (float)charact->framecount[charact->lastanimation];
	}

	//when the Event finishes reset framebegin and frameend
	// IF it was EVENT_STATIC
	if(type == EVENT_STATIC){
		obj->framebegin = 0.0;
		obj->frameend = 1.0 / (float)obj->framecount;
	}

	//when the Event finishes reset framebegin and frameend in char
	// IF it was MELEE attack
	if(type==EVENT_MELEE){
		attacker->framebegin = 0.0;
		attacker->lastanimation = lastanimation;
		attacker->frameend = 1.0 / (float)attacker->framecount[attacker->lastanimation];
	}

	//when the Event finishes reset framebegin and frameend in char
	// IF it was a HIT
	if(type==EVENT_HIT){
		victim->framebegin = 0.0;
		victim->lastanimation = lastanimation;
		victim->frameend = 1.0 / (float)victim->framecount[victim->lastanimation];
	}

	// IF it was a DEATH there's not much to do, since char won't show up on the map anymore
	if(type==EVENT_DEATH){
		//victim->framebegin = 0.0;
		//victim->lastanimation = lastanimation;
		//victim->frameend = 1.0 / (float)victim->framecount[victim->lastanimation];
	}
}

//use this function instead of passing x, y as params to constructor
//because it'd just make it too long
void Event::setviewport(int x, int y)
{
	viewportX = x;
	viewportY = y;
}

/*==============================================================
|
|
|                    EVENT_MOVEMENT
|
|
===============================================================*/



void Event::startmovement()
{
	//we have a character movement, so update pointer to character
	charact = (struct character *)ent;

	lastanimation = charact->lastanimation;

	speed = charact->speed;

	//if diagonal movement we need to adjust speed
	if(extra==EVENT_DOWNRIGHT ||
	   extra==EVENT_DOWNLEFT ||
	   extra==EVENT_UPRIGHT ||
	   extra==EVENT_UPLEFT){
								speed = charact->speed / 1.414213562373095048801;
	}

	realx=charact->realX;
	realy=charact->realY;

	//set number of total steps
	//might be modified in the near future
	//also, choose the appropriate deltax or y for delta
	
		
		if(extra==EVENT_LEFT){
			delta = deltax;
			destinx = realx-delta;
			destCol = charact->worldCol - (int)(delta/TILE_WIDTH);
			destRow = charact->worldRow;
		}else if(extra==EVENT_RIGHT){
			delta = deltax;
			destinx = realx+delta;
			destCol = charact->worldCol + (int)(delta/TILE_WIDTH);
			destRow = charact->worldRow;
		} else 	if(extra==EVENT_UP){
			delta = deltay;
			destiny = realy+delta;
			destCol = charact->worldCol;
			destRow = charact->worldRow - (int)(delta/TILE_HEIGHT);
		}else if(extra==EVENT_DOWN){
			delta = deltay;
			destiny = realy-delta;
			destCol = charact->worldCol;
			destRow = charact->worldRow + (int)(delta/TILE_HEIGHT);
		}else if(extra==EVENT_DOWNRIGHT){
		//doesn't really matter which deltax or y we use
		delta = deltay;
		destiny = realy-delta;
		destinx = realx+delta;
		destCol = charact->worldCol + (int)(delta/TILE_WIDTH);
		destRow = charact->worldRow + (int)(delta/TILE_HEIGHT);
		}else if(extra==EVENT_DOWNLEFT){
		//doesn't really matter which deltax or y we use
		delta = deltay;
		destiny = realy-delta;
		destinx = realx-delta;
		destCol = charact->worldCol - (int)(delta/TILE_WIDTH);
		destRow = charact->worldRow + (int)(delta/TILE_HEIGHT);
		}else if(extra==EVENT_UPRIGHT){
		//doesn't really matter which deltax or y we use
		delta = deltay;
		destiny = realy+delta;
		destinx = realx+delta;
		destCol = charact->worldCol + (int)(delta/TILE_WIDTH);
		destRow = charact->worldRow - (int)(delta/TILE_HEIGHT);
		}else if(extra==EVENT_UPLEFT){
		//doesn't really matter which deltax or y we use
		delta = deltay;
		destiny = realy+delta;
		destinx = realx-delta;
		destCol = charact->worldCol - (int)(delta/TILE_WIDTH);
		destRow = charact->worldRow - (int)(delta/TILE_HEIGHT);
		}


	elapsed = 0.0;

	//1 over how many frames animation at index extra <-- direction of movement
	framewidth = 1.0/(double)charact->framecount[extra];

	//set up first frame
	framebegin = 0.0;
	frameend = framebegin + framewidth;

	//update stuff inside character
	charact->framebegin = framebegin;
	charact->frameend = frameend;
	charact->lastanimation = extra;
	charact->realX = realx;
	charact->realY = realy;

}



//handle movement animation
//returns false (0) if animation has ended
int Event::updatemovement()
{

	//if he's dead, stop event
	if(charact->life<=0)
	{
		charact->alive=0;
		return 0;
	}

	delta-=speed;
	elapsed+=speed;
	if(delta>0.0)
	{
		if(extra==EVENT_LEFT)realx-=speed;
		else if(extra==EVENT_RIGHT)realx+=speed;
		else if(extra==EVENT_UP)realy+=speed;
		else if(extra==EVENT_DOWN)realy-=speed;

		else if(extra==EVENT_DOWNRIGHT){realy-=speed;realx+=speed;}
		else if(extra==EVENT_DOWNLEFT){realy-=speed;realx-=speed;}
		else if(extra==EVENT_UPRIGHT){realy+=speed;realx+=speed;}
		else if(extra==EVENT_UPLEFT){realy+=speed;realx-=speed;}
	}

	if(delta<0){
		if(extra==EVENT_LEFT)realx=destinx;
		else if(extra==EVENT_RIGHT)realx=destinx;
		else if(extra==EVENT_UP)realy=destiny;
		else if(extra==EVENT_DOWN)realy=destiny;

		else if(extra==EVENT_DOWNRIGHT){realy=destiny;realx=destinx;}
		else if(extra==EVENT_DOWNLEFT){realy=destiny;realx=destinx;}
		else if(extra==EVENT_UPRIGHT){realy=destiny;realx=destinx;}
		else if(extra==EVENT_UPLEFT){realy=destiny;realx=destinx;}
	}

	//re-index frame inside animation according to elapsed ticks
	//and also move character in world array
	if(elapsed>=TILE_WIDTH)
	{
		elapsed-=TILE_WIDTH;
		//erase char from old location on map
		w[charact->worldRow*WORLD_SIZE + charact->worldCol].charact = NULL;

		//save old location (in case of collision with another NPC)
		int oldCol = charact->worldCol;
		int oldRow = charact->worldRow;

		//calculate its new location
		if(extra == EVENT_UP)
		{
			charact->worldRow -= 1;
		}
		else if(extra == EVENT_RIGHT)
		{
			charact->worldCol += 1;
		}
		else if(extra == EVENT_LEFT)
		{
			charact->worldCol -= 1;
		}
		else if(extra == EVENT_DOWN)
		{
			charact->worldRow += 1;
		}
		else if(extra == EVENT_UPRIGHT)
		{
			charact->worldRow -= 1;
			charact->worldCol += 1;
		}
		else if(extra == EVENT_UPLEFT)
		{
			charact->worldRow -= 1;
			charact->worldCol -= 1;
		}
		else if(extra == EVENT_DOWNLEFT)
		{
			charact->worldRow += 1;
			charact->worldCol -= 1;
		}
		else if(extra == EVENT_DOWNRIGHT)
		{
			charact->worldRow += 1;
			charact->worldCol += 1;
		}

		//put it in the new location if nobody's there
		if(w[charact->worldRow*WORLD_SIZE + charact->worldCol].charact==NULL)
		{
			w[charact->worldRow*WORLD_SIZE + charact->worldCol].charact = charact;
		}
		else //we have a collision!!
		{
			//put it back, and redraw it where it was before beginning to move
			charact->worldCol = oldCol;
			charact->worldRow = oldRow;
			w[oldRow*WORLD_SIZE + oldCol].charact = charact;
			//recalculate screen coords
			charact->realX = toScreen(oldCol - viewportX)-0.1, 
			charact->realY =  -toScreen(oldRow - viewportY);

			//immediately end movement event
			return FALSE;
		}

	}

	double y = elapsed/TILE_WIDTH;
	int x = y * (double)charact->framecount[extra];


	framebegin= (double)x / (double)charact->framecount[extra];
	frameend = framebegin+framewidth;

	//update stuff inside character
	charact->framebegin = framebegin;
	charact->frameend = frameend;
	charact->lastanimation = extra;
	charact->realX = realx;
	charact->realY = realy;
	
	if(delta<=0.0){
		//we're done moving
		//before moving it to last cell of the movement, check if it's empty
		if(w[destRow*WORLD_SIZE + destCol].charact == NULL)
		{
			w[charact->worldRow*WORLD_SIZE + charact->worldCol].charact = NULL;
			charact->worldRow = destRow;
			charact->worldCol = destCol;
			w[charact->worldRow*WORLD_SIZE + charact->worldCol].charact = charact;
		}
		else //re-draw it on the last spot (it WILL not be moved as above)
		{
			charact->realX = toScreen(charact->worldCol - viewportX)-0.1, 
			charact->realY =  -toScreen(charact->worldRow - viewportY);
		}

		//event is done
		return FALSE;
	}

	//all good, and there's still more to go
	return TRUE;
}

/*==============================================================
|
|
|                    EVENT_STATIC
|
|
===============================================================*/

void Event::startstatic()
{
	//we have a object event, so update pointer to object
	obj = (struct object *)ent;
	speed = obj->speed;

	//realx=obj->realX;
	//realy=obj->realY;

	delta = deltax;
		
		
	elapsed = 0.0;

	//1 over how many frames animation at index extra <-- direction of movement
	framewidth = 1.0/(double)obj->framecount;

	//set up first frame
	framebegin = 0.0;
	frameend = framebegin + framewidth;

	//update stuff inside character
	obj->framebegin = framebegin;
	obj->frameend = frameend;
	obj->lastanimation = extra;

	//  =NEVER= update realx and realy inside objects
	//obj->realX = realx;
	//obj->realY = realy;

}



//handle static animation
//returns false (0) if animation has ended
int Event::updatestatic()
{
	delta-=speed;
	elapsed+=speed;
	
	//re-index frame inside animation according to elapsed ticks
	//and also move character in world array
	if(elapsed>=TILE_WIDTH)
	{
		elapsed-=TILE_WIDTH;
	}

	double y = elapsed/TILE_WIDTH;
	int x = y * (double)obj->framecount;


	framebegin= (double)x / (double)obj->framecount;
	frameend = framebegin+framewidth;

	//update stuff inside character
	obj->framebegin = framebegin;
	obj->frameend = frameend;
	//charact->lastanimation = extra;
	//charact->realX = realx;
	//charact->realY = realy;
	
	if(delta<=0.0){
		//event is done
		return FALSE;
	}

	//all good, and there's still more to go
	return TRUE;
}


/*==============================================================
|
|
|                    EVENT_MELEE
|
|
===============================================================*/

int Event::startmelee()
{
	//save last animation
	lastanimation = attacker->lastanimation;

	//calculate and save damage (for NOW, it's the unmodified attacker damage)
	totaldamage = attacker->damage;

	//find a suitable animation, based on position of victim relative to attacker
	int animation;
	int colDiff = attacker->worldCol - victim->worldCol;
	int rowDiff = attacker->worldRow - victim->worldRow;
	int colDist = abs(colDiff);
	int rowDist = abs(rowDiff);
	if(colDiff==0)
	{
		//victim above
		if(rowDiff>0)animation=8+2;
		//victim below
		else if(rowDiff<0)animation=8+6;
	}
	else if(rowDiff==0)
	{
		//victim to left
		if(colDiff>0)animation=8+4;
		//victim to right
		else if(colDiff<0)animation=8+0;
	}
	else
	{
		//victim is left-above
		if(colDiff>0 && rowDiff>0)animation=8+3;
		//victim is left-below
		else if(colDiff>0 && rowDiff<0)animation=8+5;
		//victim is right-above
		else if(colDiff<0 && rowDiff>0)animation=8+1;
		//victim is right-below
		else if(colDiff<0 && rowDiff<0)animation=8+7;
	}
	//========Done finding animation index==========


	//=================When melee is done, return NOT to the last state, but to same direction as melee ====
	lastanimation = animation - 8;


	speed = attacker->attackspeed;

	delta = deltax;
	halfelapsed = delta / 2.0;
	elapsed = 0.0;

	framewidth = 1.0/(double)attacker->framecount[animation];

	//set up first frame
	framebegin = 0.0;
	frameend = framebegin + framewidth;

	//update stuff inside character
	attacker->framebegin = framebegin;
	attacker->frameend = frameend;
	attacker->lastanimation = animation;

	return totaldamage;
}



//handle Melee attacks
//returns false (0) if animation has ended
int Event::updatemelee()
{
	//if he's dead, stop event
	if(attacker->life<=0)
	{
		victim->lastanimation = lastanimation;
		attacker->alive=0;
		return 0;
	}
	if(victim->life<=0)
	{
		attacker->lastanimation = lastanimation;
		victim->alive=0;
		return 0;
	}

	delta-=speed;
	elapsed+=speed;
	
	//re-index frame inside animation according to elapsed ticks
	//and also move character in world array
	if(elapsed>=TILE_WIDTH)
	{
		elapsed-=TILE_WIDTH;
	}

	double y = elapsed/TILE_WIDTH;
	int x = y * (double)attacker->framecount[attacker->lastanimation];


	framebegin= (double)x / (double)attacker->framecount[attacker->lastanimation];
	frameend = framebegin+framewidth;

	//update stuff inside character
	attacker->framebegin = framebegin;
	attacker->frameend = frameend;
	//charact->lastanimation = extra;
	//charact->realX = realx;
	//charact->realY = realy;
	
	//are we halfway through the animation ?
	if(delta < halfelapsed)
	{
		//make sure it won't happen again
		halfelapsed = -10000000.0;
		//victim loses life
		victim->life = victim->life - totaldamage;
		
		//=====TODO:  Attacker gains experience !====
	}


	if(delta<=0.0){
		//event is done

		//restore last animation index
		attacker->lastanimation = lastanimation;
		return FALSE;
	}

	//all good, and there's still more to go
	return TRUE;
}



/*==============================================================
|
|
|                    EVENT_HIT
|
|
===============================================================*/

void Event::starthit()
{
	//save last animation
	lastanimation = victim->lastanimation;

	speed = victim->recoveryspeed;

	delta = deltax;
	elapsed = 0.0;

	framewidth = 1.0/(double)victim->framecount[16];

	//set up first frame
	framebegin = 0.0;
	frameend = framebegin + framewidth;

	//update stuff inside character
	victim->framebegin = framebegin;
	victim->frameend = frameend;
	victim->lastanimation = 16;
}



//handles hits
//returns false (0) if animation has ended
int Event::updatehit()
{

	//if he's dead, stop event
	if(attacker->life<=0)
	{
		victim->lastanimation = lastanimation;
		attacker->alive=0;
		return 0;
	}
	if(victim->life<=0)
	{
		attacker->lastanimation = lastanimation;
		victim->alive=0;
		return 0;
	}

	delta-=speed;
	elapsed+=speed;
	
	//re-index frame inside animation according to elapsed ticks
	//and also move character in world array
	if(elapsed>=TILE_WIDTH)
	{
		elapsed-=TILE_WIDTH;
	}

	double y = elapsed/TILE_WIDTH;
	int x = y * (double)victim->framecount[victim->lastanimation];


	framebegin= (double)x / (double)victim->framecount[victim->lastanimation];
	frameend = framebegin+framewidth;

	//update stuff inside character
	victim->framebegin = framebegin;
	victim->frameend = frameend;
	//charact->lastanimation = extra;
	//charact->realX = realx;
	//charact->realY = realy;
	
	if(delta<=0.0){
		//event is done

		//restore last animation index
		victim->lastanimation = lastanimation;

		return FALSE;
	}

	//all good, and there's still more to go
	return TRUE;
}


/*==============================================================
|
|
|                    EVENT_DEATH
|
|
===============================================================*/

void Event::startdeath()
{
	//save last animation
	lastanimation = charact->lastanimation;

	speed = 0.003;

	delta = deltax;
	elapsed = 0.0;

	framewidth = 1.0/(double)charact->framecount[17];

	//set up first frame
	framebegin = 0.0;
	frameend = framebegin + framewidth;

	//update stuff inside character
	charact->framebegin = framebegin;
	charact->frameend = frameend;
	charact->lastanimation = 17;

	charact->alive = 0;
}



//handle hits
//returns false (0) if animation has ended
int Event::updatedeath()
{

	delta-=speed;
	elapsed+=speed;
	
	//re-index frame inside animation according to elapsed ticks
	//and also move character in world array
	if(elapsed>=TILE_WIDTH)
	{
		elapsed-=TILE_WIDTH;
	}

	double y = elapsed/TILE_WIDTH;
	int x = y * (double)charact->framecount[charact->lastanimation];


	framebegin= (double)x / (double)charact->framecount[charact->lastanimation];
	frameend = framebegin+framewidth;

	//update stuff inside character
	charact->framebegin = framebegin;
	charact->frameend = frameend;
	//charact->lastanimation = extra;
	//charact->realX = realx;
	//charact->realY = realy;
	
	if(delta<=0.0){
		//event is done

		//restore last animation index (although unnecessary since it dies!)
		charact->lastanimation = lastanimation;

		//erase character from world !
		w->charact->alive = 0;
		w->charact = NULL;

		return FALSE;
	}

	//all good, and there's still more to go
	return TRUE;
}