/*
	Event describes different animation (and not only) aspects.
	It provides a means of controlling the actions of more than one entity,
	thus allowing for real-time gameplay.
 */

#ifndef __EVENT__
#define __EVENT__

#include "Ticker.h"
#include "char.h"
#include "cell.h"
#include "Constants.h"

//a character movement event
#define EVENT_MOVEMENT 0
//an animation on the map (such as a burning fire)
#define EVENT_STATIC 1
//an animation of a char hitting another char
#define EVENT_MELEE 2
//an animation of a char being hit, and recovering from the hit
#define EVENT_HIT 3
//a dying character
#define EVENT_DEATH 4

//Different extra information about the event type
#define EVENT_LEFT 4
#define EVENT_DOWN 6
#define EVENT_UP 2
#define EVENT_RIGHT 0

#define EVENT_UPLEFT 3
#define EVENT_UPRIGHT 1
#define EVENT_DOWNRIGHT 7
#define EVENT_DOWNLEFT 5

class Event
{
//private:
public:

	//pointer to entity that generated an event
	//TODO: change void to whatever Andrei calls it
	void *ent;

	int viewportX, viewportY;

	//world array of cells
	struct cell *w;

	//movement left for this entity
	double deltax, deltay;
	//for choosing just one, for simple versions of game
	double delta;

	double elapsed;
	//what kind of event happened to ent ?
	int type;

	//extra information such as direction of movement
	int extra;

	//Number of frames in the animation to be carried out
	int frames;

	//width of one frame
	double framewidth;

	//screen coords
	double destinx;
	double destiny;

	//world coords
	int destCol, destRow;

	//rendering-specific stuff here

	//animation frame selectors
	double framebegin;
	double frameend;
	//DON'T need gamex, gamey, since those are updated first
	//through ent->  as soon the event is good to start

	//actual coordinates of presently rendered QUAD are needed though
	double realx, realy;
	//speed of entity in real distance/tick
	double speed;

	//in case this event is generated for a character
	struct character *charact;

	//in case this is an EVENT_MELEE
	struct character *attacker, *victim;

	//in case this event is a static event
	struct object *obj;

	//in case of EVENT_MELEE, we need to save some values
	int totaldamage, lastanimation;

	//this var is set to half the event total time, so life can be decreased
	//halfway through the animation in case of an attack
	double halfelapsed;

	//timing specific variables, needed for animation timing
	//clock will be sent from main program, in order to have them sync'd
	Ticker *clock;

public:
	Event();
	//a constructor for something that doesn't need to move.
	Event(void *entity, int eventtype, int extrainfo, Ticker *cl);
	//more comprehensive, allows for movement and also static object animations
	Event(struct cell *c, void *entity, int eventtype, int extrainfo, double displacementx, double displacementy);
	//used for EVENT_MELEE and EVENT_HIT so far.
	Event(struct character *attack, struct character *target, int eventtype, float displacement, int damage);
	//used for EVENT_DEATH
	Event(struct cell *c, struct character *chrct, int eventtype, float displacement);
	
	~Event();

	void setviewport(int x, int y);

	void startmovement();
	int updatemovement();

	void startstatic();
	int updatestatic();

	int startmelee();
	int updatemelee();

	void starthit();
	int updatehit();

	void startdeath();
	int updatedeath();
	
};

#endif