/*
	Trigger is a simple struct that enable scripting.
	Each  <cell> contains a Trigger.
	in case we don't want a trigger somewhere, we simply set its type to TRIGGER_NONE
	otherwise, we use the predefined types below.
*/

#ifndef _TRIGGER_
#define _TRIGGER_


//Trigger types

//no trigger (do nothing)
#define TRIGGER_NONE -1
//triggered by the death of a character or a group of characters (doesn't matter if onscree or not)
#define TRIGGER_DEATH 1
//triggered by the picking up of an item or just having it in the inventory
#define TRIGGER_PICKUP 2
//triggered by a character being alive
#define TRIGGER_ALIVE 3
//triggered by a character walking over the cell with this trigger
#define TRIGGER_WALKOVER 4


//add something to character's inventory (as a reward)
#define TRIGGER_ACTION_REWARD 1
//change the alignment of a character
//the new alignment is specified in Trigger.direction
#define TRIGGER_ACTION_CONVERT 2
//make a character walk in a certain direction, for a certain distance
#define TRIGGER_ACTION_MOVE 3
//kill a character
#define TRIGGER_ACTION_KILL 4
//vanish an object
#define TRIGGER_ACTION_VANISH 5
//open a door by changing its texture and also by making that tile passable
#define TRIGGER_ACTION_OPEN 6


//Trigger longevity
//make this happen just once
#define TRIGGER_LONGEVITY_ONCE 1
//make it happen whenever the condition is true
#define TRIGGER_LONGEVITY_ALWAYS 2

//Trigger sources (which character triggered it) and targets (who will be affected)
#define TRIGGER_NOBODY -1
#define TRIGGER_ALL -100

#define MAXTRIGGERSOURCES 8

struct Trigger
{
	//where is this trigger located ?
	//   OR the coords of tile we need to  TRIGGER_ACTION_OPEN
	int worldRow, worldCol;

	//contains indices of sources in global characters[] array
	int sources[MAXTRIGGERSOURCES];

	//contains indices of targets in global characters[] array
	//    OR the index of textures on a tile in case of TRIGGER_ACTION_OPEN as such:
	//               0 = OPEN floor, 1=OPEN down facing texture, etc.
	int targets[MAXTRIGGERSOURCES];

	//can be NONE, DEATH, PICKUP, ALIVE, etc.
	int cause;
	
	//can be REWARD, CONVERT, MOVE, VANISH, etc.
	int effect;

	//can be ONCE or ALWAYS
	int longevity;

	//direction of movement, in case of TRIGGER_ACTION_MOVE
	//   OR  the new alignment in case of TRIGGER_ACTION_CONVERT
	//   OR  the index of the new texture, in case of a TRIGGER_ACTION_OPEN
	int direction;

	//distance of movement
	float distance;
};


#endif