//
// Project One is copyrighted by University of Toronto's Game Programming Club, 3004
// This is NOT public software; it is private and 
// is meant exclusively for the members of the above mentioned organization.
// Viewing, copying, altering, performed by non-members is strictly prohibited.
//

#include <math.h>
#include <stdarg.h>
#include <windows.h>											// Header File For Windows
#include <stdio.h>												// Header File For Standard Input/Output
#include <gl\gl.h>												// Header File For The OpenGL32 Library
#include <gl\glu.h>												// Header File For The GLu32 Library
#include <string.h>
#include "Texture.h"											// Header File Containing Our Texture Structure ( NEW )

int onscreen=0;

//our ingame clock
#include "Ticker.h"
#include "Event.h"
#include "Constants.h"
#include "cell.h"
#include "char.h"
#include "TextureIndices.h" //indices for global tex array
#include "screen.h"
#include "object.h"
#include "Trigger.h"

#pragma comment( lib, "opengl32.lib" )
#pragma comment( lib, "glu32.lib" )	
#pragma comment( lib, "glaux.lib" )	
#pragma comment( lib, "winmm.lib" )


HDC			hDC=NULL;											// Private GDI Device Context
HGLRC		hRC=NULL;											// Permanent Rendering Context
HWND		hWnd=NULL;											// Holds Our Window Handle
HINSTANCE	hInstance;											// Holds The Instance Of The Application

//MOUSE
GLdouble posX, posY, posZ, mouseX, mouseY;
int mouseCol, mouseRow;

//FONT
GLuint	base;				// Base Display List For The Font Set

bool	keys[256];												// Array Used For The Keyboard Routine
bool	active=TRUE;											// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;										// Fullscreen Flag Set To Fullscreen Mode By Default

//Seba
double window_height;
double window_width;

//frame per second control
DWORD start_time_andrei = GetTickCount();

//bring the screen to [min_width, max_width] horizontally, and [-1.0, 1.0] vertically
double max_width;
double min_width;

//coords of viewport in the map array
int viewportX=0;
int viewportY=0;

//translation of main game display to the left
double translatefactor = 0.0;


int currenttex = 0;
int currentori = 0;
float currentzoom = 0.0;
float zoom = 0.0;
int lastobjectindex = -1;


//a left-over.........might be useful
Ticker *ticks;

//shows whether the mouse is clicked or not
int clicked=0;

Texture tex[MAXTEXTURES];
Texture corner;
Texture mouse;

//holds level cells
struct cell w[WORLD_SIZE][WORLD_SIZE];

//holds all events happening on the screen at any given moment
Event* events[MAXEVENTS];

//holds all characters on this map (including player)
struct character characters[MAXCHARACTERS];

//holds all objects on this map
struct object objects[MAXOBJECTS];

//hold all triggers on this map
struct Trigger triggers[MAXTRIGGERS];

/*
	========================
	Global variables
	========================
*/
int player_attacking=0;
int player_last_target=0;

/*
    ========================
	Header definitions below
	========================
*/

void save_map();
void scroll_viewport();

void add_object();

double movement_getfarthest(int direction, double maxdistance, int charno);
int attempt_movement(int charnum, int direction, float maxdistance);
int attempt_melee_attack(int att, int vic);
int npc_findtarget(struct character *charact);
void npc_follow();
int mouse_handle();
int act_trigger(struct Trigger *t);
int check_trigger(struct Trigger *t);
void check_triggers();
int activate_objects();
void grim_reaper();

void initResources();
bool LoadTGA(Texture *, char *);
void SetMouse(int x, int y);
int GameFrame(GLvoid);
int GamePreFrame(GLvoid);
int GamePostFrame(GLvoid);
void drawmouse(double x, double y);
void drawcorner();

//leftTex and rightTex are tex coordinates
void drawtile(double x, double y, int position, double zoom, double leftTex, double rightTex);
//draw all tiles on a cell, in a specific order
void drawcell(double x, double y, struct cell c);
//draw all objects on a cell in order
void drawobject(double x, double y, struct cell c);
//draw the character on this tile
void drawcharacter(double x, double y, struct cell c);
//draw 30 by 30 map cells, first tiles, then objects
void drawmap();

LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

/*
	======================
	End header definitions
	======================
*/


//===========FONT=============
GLvoid BuildFont(GLvoid)								// Build Our Bitmap Font
{
	HFONT	font;										// Windows Font ID
	HFONT	oldfont;									// Used For Good House Keeping

	base = glGenLists(96);								// Storage For 96 Characters

	font = CreateFont(	-12,							// Height Of Font
						0,								// Width Of Font
						0,								// Angle Of Escapement
						0,								// Orientation Angle
						FW_BOLD,						// Font Weight
						FALSE,							// Italic
						FALSE,							// Underline
						FALSE,							// Strikeout
						ANSI_CHARSET,					// Character Set Identifier
						OUT_TT_PRECIS,					// Output Precision
						CLIP_DEFAULT_PRECIS,			// Clipping Precision
						ANTIALIASED_QUALITY,			// Output Quality
						FF_DONTCARE|DEFAULT_PITCH,		// Family And Pitch
						"Bookman Old Style");					// Font Name

	oldfont = (HFONT)SelectObject(hDC, font);           // Selects The Font We Want
	wglUseFontBitmaps(hDC, 32, 96, base);				// Builds 96 Characters Starting At Character 32
	SelectObject(hDC, oldfont);							// Selects The Font We Want
	DeleteObject(font);									// Delete The Font
}

GLvoid KillFont(GLvoid)									// Delete The Font List
{
	glDeleteLists(base, 96);							// Delete All 96 Characters
}

GLvoid glPrint(const char *fmt, ...)					// Custom GL "Print" Routine
{
	char		text[256];								// Holds Our String
	va_list		ap;										// Pointer To List Of Arguments

	if (fmt == NULL)									// If There's No Text
		return;											// Do Nothing

	va_start(ap, fmt);									// Parses The String For Variables
	    vsprintf(text, fmt, ap);						// And Converts Symbols To Actual Numbers
	va_end(ap);											// Results Are Stored In Text

	glPushAttrib(GL_LIST_BIT);							// Pushes The Display List Bits
	glListBase(base - 32);								// Sets The Base Character to 32
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text
	glPopAttrib();										// Pops The Display List Bits
}

//==================== END FONT ======================

int LoadGLTextures()											// Load Bitmaps And Convert To Textures
{
	// Load The Bitmap, Check For Errors.
	for(int texcount=0;texcount<MAXTEXTURES;texcount++)
	{
		char filename[128];
		sprintf(filename, "Data/%d.tga", texcount);
		if(!LoadTGA(&tex[texcount], filename))
		{
			return FALSE;
		}
	}
		

	for (int loop=0; loop<MAXTEXTURES; loop++)						// Loop Through Both Textures
	{
		// Typical Texture Generation Using Data From The TGA ( CHANGE )
		glGenTextures(1, &tex[loop].texID);				// Create The Texture ( CHANGE )
		glBindTexture(GL_TEXTURE_2D, tex[loop].texID);
		glTexImage2D(GL_TEXTURE_2D, 0, tex[loop].bpp / 8, tex[loop].width, tex[loop].height, 0, tex[loop].type, GL_UNSIGNED_BYTE, tex[loop].imageData);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		
		if (tex[loop].imageData)						// If Texture Image Exists ( CHANGE )
		{
			free(tex[loop].imageData);					// Free The Texture Image Memory ( CHANGE )
		}
	}

	//===========load corner of screen===============
	LoadTGA(&corner, "Data/corner.tga");
	glGenTextures(1, &corner.texID);				// Create The Texture ( CHANGE )
	glBindTexture(GL_TEXTURE_2D, corner.texID);
	glTexImage2D(GL_TEXTURE_2D, 0, corner.bpp / 8, corner.width, corner.height, 0, corner.type, GL_UNSIGNED_BYTE, corner.imageData);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	if (corner.imageData)						// If Texture Image Exists ( CHANGE )
		{
		free(corner.imageData);					// Free The Texture Image Memory ( CHANGE )
		}
	//================================================

	//===========load mouse pointer===============
	LoadTGA(&mouse, "Data/mouse.tga");
	glGenTextures(1, &mouse.texID);				// Create The Texture ( CHANGE )
	glBindTexture(GL_TEXTURE_2D, mouse.texID);
	glTexImage2D(GL_TEXTURE_2D, 0, mouse.bpp / 8, mouse.width, mouse.height, 0, mouse.type, GL_UNSIGNED_BYTE, mouse.imageData);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	if (mouse.imageData)						// If Texture Image Exists ( CHANGE )
		{
		free(mouse.imageData);					// Free The Texture Image Memory ( CHANGE )
		}
	//================================================

	return TRUE;												// Return The Status
}

GLvoid ReSizeGLScene(GLsizei width, GLsizei height)				// Resize And Initialize The GL Window
{
	if (height==0)												// Prevent A Divide By Zero By
	{
		height=1;												// Making Height Equal One
	}

	window_height = height;
	window_width = width;

	max_width = window_width / window_height;
	min_width = -1.0*max_width;

	glViewport(0,0,width,height);								// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);								// Select The Projection Matrix
	glLoadIdentity();											// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);									// Select The Modelview Matrix
	glLoadIdentity();											// Reset The Modelview Matrix
}

int InitGL(GLvoid)												// All Setup For OpenGL Goes Here
{
	if (!LoadGLTextures())										// Jump To Texture Loading Routine ( NEW )
	{
		return FALSE;											// If Texture Didn't Load Return FALSE
	}

	glEnable(GL_TEXTURE_2D);									// Enable Texture Mapping ( NEW )
	glShadeModel(GL_SMOOTH);									// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);						// Black Background
	glClearDepth(1.0f);											// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);									// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);										// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Really Nice Perspective Calculations

	BuildFont();

	return TRUE;												// Initialization Went OK
}


GLvoid KillGLWindow(GLvoid)										// Properly Kill The Window
{
	if (fullscreen)												// Are We In Fullscreen Mode?
	{
		ChangeDisplaySettings(NULL,0);							// If So Switch Back To The Desktop
		ShowCursor(TRUE);										// Show Mouse Pointer
	}

	if (hRC)													// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))							// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))								// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;												// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))							// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;												// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))							// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;												// Set hWnd To NULL
	}

	if (!UnregisterClass("OpenGL",hInstance))					// Are We Able To Unregister Class
	{
		MessageBox(NULL,"Could Not Unregister Class.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;											// Set hInstance To NULL
	}
}

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;									// Holds The Results After Searching For A Match
	WNDCLASS	wc;												// Windows Class Structure
	DWORD		dwExStyle;										// Window Extended Style
	DWORD		dwStyle;										// Window Style
	RECT		WindowRect;										// Grabs Rectangle Upper Left / Lower Right Values
	WindowRect.left=(long)0;									// Set Left Value To 0
	WindowRect.right=(long)width;								// Set Right Value To Requested Width
	WindowRect.top=(long)0;										// Set Top Value To 0
	WindowRect.bottom=(long)height;								// Set Bottom Value To Requested Height

	fullscreen=fullscreenflag;									// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	if (fullscreen)												// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;								// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP;										// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	//else
	if(!fullscreen)
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW;							// Windows Style
	}

	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);	// Adjust Window To True Requested Size

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,							// Extended Style For The Window
								"OpenGL",							// Class Name
								title,								// Window Title
								dwStyle |							// Defined Window Style
								WS_CLIPSIBLINGS |					// Required Window Style
								WS_CLIPCHILDREN,					// Required Window Style
								0, 0,								// Window Position
								WindowRect.right-WindowRect.left,	// Calculate Window Width
								WindowRect.bottom-WindowRect.top,	// Calculate Window Height
								NULL,								// No Parent Window
								NULL,								// No Menu
								hInstance,							// Instance
								NULL)))								// Dont Pass Anything To WM_CREATE
	{
		KillGLWindow();											// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}

	
	static	PIXELFORMATDESCRIPTOR pfd=							// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),							// Size Of This Pixel Format Descriptor
		1,														// Version Number
		PFD_DRAW_TO_WINDOW |									// Format Must Support Window
		PFD_SUPPORT_OPENGL |									// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,										// Must Support Double Buffering
		PFD_TYPE_RGBA,											// Request An RGBA Format
		bits,													// Select Our Color Depth
		0, 0, 0, 0, 0, 0,										// Color Bits Ignored
		1,														// No Alpha Buffer
		0,														// Shift Bit Ignored
		0,														// No Accumulation Buffer
		0, 0, 0, 0,												// Accumulation Bits Ignored
		16,														// 16Bit Z-Buffer (Depth Buffer)  
		0,														// No Stencil Buffer
		0,														// No Auxiliary Buffer
		PFD_MAIN_PLANE,											// Main Drawing Layer
		0,														// Reserved
		0, 0, 0													// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))										// Did We Get A Device Context?
	{
		KillGLWindow();											// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))				// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();											// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))					// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();											// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))							// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();											// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))								// Try To Activate The Rendering Context
	{
		KillGLWindow();											// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);									// Show The Window
	SetForegroundWindow(hWnd);									// Slightly Higher Priority
	SetFocus(hWnd);												// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);								// Set Up Our Perspective GL Screen

	if (!InitGL())												// Initialize Our Newly Created GL Window
	{
		KillGLWindow();											// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}

	return TRUE;												// Success
}

LRESULT CALLBACK WndProc(	HWND	hWnd,						// Handle For This Window
							UINT	uMsg,						// Message For This Window
							WPARAM	wParam,						// Additional Message Information
							LPARAM	lParam)						// Additional Message Information
{
	switch (uMsg)												// Check For Windows Messages
	{
		case WM_ACTIVATE:										// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))								// Check Minimization State
			{
				active=TRUE;									// Program Is Active
			}
			else
			{
				active=FALSE;									// Program Is No Longer Active
			}

			return 0;											// Return To The Message Loop
		}

		case WM_SYSCOMMAND:										// Intercept System Commands
		{
			switch (wParam)										// Check System Calls
			{
				case SC_SCREENSAVE:								// Screensaver Trying To Start?
				case SC_MONITORPOWER:							// Monitor Trying To Enter Powersave?
				return 0;										// Prevent From Happening
			}
			break;												// Exit
		}

		case WM_CLOSE:											// Did We Receive A Close Message?
		{
			PostQuitMessage(0);									// Send A Quit Message
			return 0;											// Jump Back
		}

		case WM_KEYDOWN:										// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;								// If So, Mark It As TRUE
			return 0;											// Jump Back
		}

		case WM_KEYUP:											// Has A Key Been Released?
		{
			keys[wParam] = FALSE;								// If So, Mark It As FALSE
			return 0;											// Jump Back
		}

		//for mouse
		case WM_LBUTTONDOWN:
		{
			clicked = 1;
			return 0;
		}
		case WM_LBUTTONUP:
		{
			clicked = 0;
			player_attacking = 0;
			return 0;
		}



		case WM_RBUTTONDOWN:
		{
			add_object();
			return 0;
		}


		case WM_SIZE:											// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));		// LoWord=Width, HiWord=Height
			return 0;											// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

int WINAPI WinMain(	HINSTANCE	hInstance,						// Instance
					HINSTANCE	hPrevInstance,					// Previous Instance
					LPSTR		lpCmdLine,						// Command Line Parameters
					int			nCmdShow)						// Window Show State
{
	MSG		msg;												// Windows Message Structure
	BOOL	done=FALSE;											// Bool Variable To Exit Loop


	int w, h, bpp, full;

	FILE *fp;
	fp = fopen("Config/default.ini", "r");
	fscanf(fp, "%d\n%d\n%d\n%d", &w, &h, &bpp, &full);
	fclose(fp);

	//vsync disable
	BOOL (APIENTRY *wglSwapIntervalEXT)(int);
	wglSwapIntervalEXT=( BOOL (APIENTRY*)(int) )wglGetProcAddress("wglSwapIntervalEXT");
	if (wglSwapIntervalEXT)wglSwapIntervalEXT(1);


	//no full for now.
	fullscreen=FALSE; 
	// Create Our OpenGL Window
	if (!CreateGLWindow("",w, h, bpp, /*fullscreen*/full))
	{
		return 0;												// Quit If Window Was Not Created
	}

	initResources();

	//start ticking
	//ticks = (Ticker*)malloc(sizeof(Ticker));
	//default to 60 ticks/second
	//ticks->start(30.0);

	while(!done)												// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))				// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)							// Have We Received A Quit Message?
			{
				done=TRUE;										// If So done=TRUE
			}
			else												// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);							// Translate The Message
				DispatchMessage(&msg);							// Dispatch The Message
			}
		}
		else													// If There Are No Messages
		{
			if(keys[VK_ESCAPE])
			{
				done=TRUE;
			}


			


			//reset keypresses if minimized
			if(!active)
			{
				for(int i=0;i<256;i++){
					keys[i]=FALSE;
				}
			}

			if (active)	// Active?  Was There A Quit Received?
			{
				if(!GamePreFrame()){
					done=TRUE;
				}else if(!GameFrame()){
					done=TRUE;
				}else if(!GamePostFrame()){
					done=TRUE;
				}
			}
			
			if(!done)
			{
				SwapBuffers(hDC);								// Swap Buffers (Double Buffering)
			}

			/*if (keys[VK_F1])									// Is F1 Being Pressed?
			{
				keys[VK_F1]=FALSE;								// If So Make Key FALSE
				KillGLWindow();									// Kill Our Current Window
				fullscreen=!fullscreen;							// Toggle Fullscreen / Windowed Mode
				// Recreate Our OpenGL Window
				if (!CreateGLWindow("",640,480,32,fullscreen))
				{
					return 0;									// Quit If Window Was Not Created
				}
			}*/
			
			//SEBA: key handling here or even better in PreFrame()

			if (keys[VK_RIGHT] && viewportX<WORLD_SIZE-1)
			{
				viewportX ++;
			}
			if (keys[VK_LEFT] && viewportX>0)
			{
				viewportX --;
			}
			if (keys[VK_UP] && viewportY>0)
			{
				viewportY --;
			}
			if (keys[VK_DOWN] && viewportY<WORLD_SIZE-1)
			{
				viewportY ++;
			}
//==============================TEXTURES============================
			if (keys['Q'])
			{
				static int counter = 0;
				counter++;
				if(counter%3==0)currenttex = currenttex-1;

				if(currenttex<0)currenttex=MAXTEXTURES-1;
			}
			if (keys['W'])
			{
				static int counter = 0;
				counter++;
				if(counter%3==0)currenttex = currenttex+1;

				if(currenttex>MAXTEXTURES-1)currenttex=0;
			}

//==============================POSITIONING============================
			if (keys['A'])
			{
				static int counter = 0;
				counter++;
				if(counter%3==0)currentori = currentori-1;

				if(currentori<0)currentori=6;
			}
			if (keys['S'])
			{
				static int counter = 0;
				counter++;
				if(counter%3==0)currentori = currentori+1;

				if(currentori>6)currentori=0;
			}

//==========================MAP SAVING======================
			if(keys[VK_F5])
			{
				save_map();
			}






		}
	}

	// Shutdown
	KillGLWindow();												// Kill The Window
	return (msg.wParam);										// Exit The Program
}

int GameFrame(GLvoid)											// Here's Where We Do All The Drawing
{

	//MAN's calculations, etc. were done above

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			// Clear The Screen And The Depth Buffer
	glLoadIdentity();											// Reset The Modelview Matrix
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	
	glScalef(window_height/window_width, 1.0, 1.0);
	glScalef(0.75, 0.5, 1.0);


	glTranslatef(0.0, 4*TILE_WIDTH, 0.0);

	//save because text will have to be written
	glPushMatrix();

	glRotatef(45.0, 0.0, 0.0,1.0);

	glTranslatef(-translatefactor, translatefactor, 0.0);

	static float noapte = 2.0;

	glAlphaFunc( GL_GREATER, 0.5 );
	glEnable( GL_ALPHA_TEST );

	glColor3f(1.0, 1.0, 1.0);

	drawmap();

	glColor3f(1.0, 1.0, 1.0);

//=============================================
//	MATRIX IS POPPED -> NO ISOMETRY HEREAFTER
//=============================================

glPopMatrix();

//drawcorner();


//display current texture
glBindTexture(GL_TEXTURE_2D, tex[currenttex].texID);			// ( CHANGE )
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(-1.70,-1.0, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(-1.40,-1.0, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(-1.40,-1.30, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(-1.70,-1.30, 0.0f);
glEnd();


//display current cell info
struct cell c;

if(mouseRow>-1 && mouseCol>-1)
	c = w[mouseRow][mouseCol];

int cur=-1;
for(float ff=-1.2;ff>=-2.5;ff-=0.2)
{
	char a[32];
	
	cur++;

	if(cur==0)strcpy(a, "Floor");
	if(cur==1)strcpy(a, "Up");
	if(cur==2)strcpy(a, "Right");
	if(cur==3)strcpy(a, "Left");
	if(cur==4)strcpy(a, "Down");
	if(cur==5)strcpy(a, "Roof");
	if(cur==6)strcpy(a, "FLAT");

	if(mouseRow>-1 && mouseCol>-1)
	{
	glBindTexture(GL_TEXTURE_2D, tex[c.tiles[cur].index].texID);			// ( CHANGE )
	glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(1.60,ff+0.15, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(1.75,ff+0.15, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(1.75,ff, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(1.60,ff, 0.0f);
	glEnd();
	}
	//DISPLAY bogus polygon with clear texture
	glBindTexture(GL_TEXTURE_2D, tex[TEX_CLEAR].texID);			// ( CHANGE )
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(0.0,0.0, 0.0f);
	glEnd();

	glDisable( GL_ALPHA_TEST );
	glColor3f(1.0, 1.0, 1.0);

	glRasterPos2f(1.45,ff+0.08);
	glPrint(a);

	glColor3f(0.0, 1.0, 1.0);
	glRasterPos2f(1.25,-0.9);
	glPrint("Current Tile Info");
	glColor3f(1.0, 1.0, 1.0);

	glRasterPos2f(1.25,-0.97);
	glPrint("(%d,%d)",mouseRow,mouseCol);

	glRasterPos2f(1.45,-0.97);
	glPrint("Passable: ");

	if(mouseRow>-1 && mouseCol>-1){
	glRasterPos2f(1.67,-0.97);
	if(c.passable)
		glPrint("YES");
	else
		glPrint("NO");
	}

	glEnable(GL_ALPHA_TEST);

}
//end current cell info



	//draw mouse pointer with colours accordingly
	//change colour if mouse hovering over this char (but don't do it for main guy)
		if(mouseRow>-1 && mouseCol>-1 && 
		   w[mouseRow][mouseCol].charact != &characters[0] &&
		   w[mouseRow][mouseCol].charact != NULL)
		{
			if(w[mouseRow][mouseCol].charact->alignment==FRIENDLY)
			{
				glColor3f(0.3, 1.0, 0.3);
			}
			else if(w[mouseRow][mouseCol].charact->alignment==HOSTILE)
			{
				glColor3f(1.0, 0.3, 0.3);
			}
		}
	drawmouse(posX, posY);
	
	

	glColor3f(1.0, 1.0, 1.0);
	//min width test
	
	//DISPLAY bogus polygon with clear texture
	glBindTexture(GL_TEXTURE_2D, tex[TEX_CLEAR].texID);			// ( CHANGE )
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(0.0,0.0, 0.0f);
	glEnd();


	return TRUE;												// Keep Going
}



/*==============================================================
|
|
|              PREFRAME AND POSTFRAME
|
|
===============================================================*/

int GamePreFrame()
{
	POINT p;
	GetCursorPos(&p);
	SetMouse(p.x, p.y);

	//if mouse is outside, make both mouseCol and mouseRow negative for future checks
	if(toWorld(mouseX)>=0 && toWorld(mouseY)>=0)
	{
		mouseCol = viewportX + toWorld(mouseX);
		mouseRow = viewportY + 29-toWorld(mouseY);
	}else
	{
		mouseCol = -1;
		mouseRow = -1;
	}

	//only handle mouse if mouse is over play area and was clicked
	if(clicked && mouseCol>=0 && mouseRow>=0)
		mouse_handle();


	activate_objects();



	int count;
//update all events, checking for the ones that finished
for(count = 0; count<MAXEVENTS; count++)
{
	if(events[count]!=NULL)
	{
		int result = -1;
		if(events[count]->type==EVENT_MOVEMENT)
				result = events[count]->updatemovement();

		if(events[count]->type==EVENT_STATIC)
				result = events[count]->updatestatic();

		if(events[count]->type==EVENT_MELEE)
				result = events[count]->updatemelee();

		if(events[count]->type==EVENT_HIT)
				result = events[count]->updatehit();

		if(events[count]->type==EVENT_DEATH)
		{
				result = events[count]->updatedeath();
		}

		//this event has finished
		//result will be anything but 0 if this event doesn't exist or it hasn't finished
		if(!result){		
			delete events[count];
			events[count]=NULL;
		}
	}
}
	

if(mouseRow>-1 && mouseCol>-1)
{
//==============================ZOOM============================
			if (keys['Z'] || keys['C'])
			{
				static int counter = 0;
				float amount = 0.5;

				if(keys['C'])amount = 10.0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].tiles[currentori].zoom = w[mouseRow][mouseCol].tiles[currentori].zoom-amount;

				if(w[mouseRow][mouseCol].tiles[currentori].zoom<0.0)w[mouseRow][mouseCol].tiles[currentori].zoom=0.0;
			}
			if (keys['X'] || keys['V'])
			{
				static int counter = 0;
				float amount = 0.5;

				if(keys['V'])amount = 10.0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].tiles[currentori].zoom = w[mouseRow][mouseCol].tiles[currentori].zoom+amount;
			}

			if(keys['1'])
			{
				w[mouseRow][mouseCol].passable=0;
			}
			if(keys['2'])
			{
				w[mouseRow][mouseCol].passable=1;
			}


			//erase cell
			if(keys[VK_BACK])
			{
				//start from 1, meaning DON'T ERASE FLOOR !
				for(int i=1;i<7;i++)
				{
					w[mouseRow][mouseCol].tiles[i].zoom = 0.0;
					//clear texture
					w[mouseRow][mouseCol].tiles[i].index = 0;
					w[mouseRow][mouseCol].passable = 1;
				}

				//clear object if any
				if(w[mouseRow][mouseCol].obj!=NULL)
				{
					w[mouseRow][mouseCol].obj->worldCol=-1;
					w[mouseRow][mouseCol].obj->worldRow=-1;
					w[mouseRow][mouseCol].obj = NULL;
				}
			}

			//this if for objects stirctly

			//pageup/pagedown control number of frames of an object
			if(keys[VK_PRIOR] && w[mouseRow][mouseCol].obj!=NULL)
			{
				static int counter = 0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].obj->framecount++;
			}
			if(keys[VK_NEXT] && w[mouseRow][mouseCol].obj!=NULL)
			{
				static int counter = 0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].obj->framecount--;
				if(w[mouseRow][mouseCol].obj->framecount<0)
					w[mouseRow][mouseCol].obj->framecount=0;
			}


			//home/end control speed of animation
			if(keys[VK_HOME] && w[mouseRow][mouseCol].obj!=NULL)
			{
				static int counter = 0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].obj->speed+=0.001;
			}
			if(keys[VK_END] && w[mouseRow][mouseCol].obj!=NULL)
			{
				static int counter = 0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].obj->speed-=0.001;
				if(w[mouseRow][mouseCol].obj->speed<0.0005)
					w[mouseRow][mouseCol].obj->speed=0.0005;
			}

			//insert/delete control zoom of an object
			if(keys[VK_INSERT] && w[mouseRow][mouseCol].obj!=NULL)
			{
				static int counter = 0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].obj->zoom+=0.5;
			}
			if(keys[VK_DELETE] && w[mouseRow][mouseCol].obj!=NULL)
			{
				static int counter = 0;

				counter++;
				if(counter%3==0)
					w[mouseRow][mouseCol].obj->zoom-=0.5;
				if(w[mouseRow][mouseCol].obj->zoom<0.000)
					w[mouseRow][mouseCol].obj->zoom=0.05;
			}
}//end if mouserov>-1 && mousecol>-1


if(mouseRow>-1 && mouseCol>-1)
	currentzoom = w[mouseRow][mouseCol].tiles[currentori].zoom;


	return TRUE;
}


/*=======================================
|
|          POSTFRAME
========================================*/
int GamePostFrame()
{

	/*
		DEBUG MESSAGES
	*/
	glDisable( GL_ALPHA_TEST );

glColor3f(0.0, 1.0, 1.0);

glRasterPos2f(-1.70,-1.42);
glPrint("Current texture");




glRasterPos2f(-1.70,-1.72);
glPrint("Current orientation");


glRasterPos2f(-1.70,-2.02);
glPrint("Current zoom");


glRasterPos2f(1.10,1.52);
glPrint("Current OBJECT: ");

glColor3f(1.0, 1.0, 1.0);
if(mouseRow>-1 && mouseCol>-1 && w[mouseRow][mouseCol].obj==NULL)
{
	glRasterPos2f(1.50,1.52);
	glPrint("NULL");
}
else if(mouseRow>-1 && mouseCol>-1)
{
	glRasterPos2f(1.10,1.42);
	glPrint("(%d,%d)",mouseRow, mouseCol);

	glRasterPos2f(1.40,1.42);
	glPrint("Zoom %3.3f",w[mouseRow][mouseCol].obj->zoom);

	glRasterPos2f(1.40,1.32);
	glPrint("Framecount %d",w[mouseRow][mouseCol].obj->framecount);

	glRasterPos2f(1.40,1.22);
	glPrint("Speed %3.3f",w[mouseRow][mouseCol].obj->speed);


}




glColor3f(1.0, 1.0, 1.0);

glRasterPos2f(-1.35,-1.15);
glPrint("#%d", currenttex);


glRasterPos2f(-1.70,-1.92);
glPrint("%3.2f", currentzoom);

//orientation
glRasterPos2f(-1.70,-1.63);
if(currentori==TILE_FLOOR)glPrint("FLOOR");
if(currentori==TILE_UP)glPrint("UP");
if(currentori==TILE_DOWN)glPrint("DOWN");
if(currentori==TILE_LEFT)glPrint("LEFT");
if(currentori==TILE_RIGHT)glPrint("RIGHT");
if(currentori==TILE_ROOF)glPrint("ROOF");
if(currentori==6)glPrint("FLAT RENDERED(Object)");


//DISPLAY bogus polygon with clear texture
	glBindTexture(GL_TEXTURE_2D, tex[TEX_CLEAR].texID);			// ( CHANGE )
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(0.0,0.0, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(0.0,0.0, 0.0f);
	glEnd();

/*	glRasterPos2f(-1.77, 1.55);
	glPrint("winX=%3.2f winY=%3.2f",posX, posY);
*/

	int ob=0;
	for(int ii=0;ii<MAXOBJECTS;ii++)
		if(objects[ii].worldCol>=0)ob++;


	glRasterPos2f(-1.77, 1.45);
	glPrint("viewportY=%d viewportX=%d",viewportY, viewportX);

	glRasterPos2f(-1.77, 1.35);
	glPrint("mouseRow=%d   mouseCol=%d", mouseRow, mouseCol);

	glRasterPos2f(-1.77, 1.55);
	glPrint("Total valid objects %d", ob);

	glRasterPos2f(-1.77, 1.50);
	glPrint("Available objects %d", MAXOBJECTS-lastobjectindex-1);

	while((GetTickCount() - start_time_andrei) < 27.0 );
	start_time_andrei = GetTickCount();

	return TRUE;
}


/*==============================================================
|
|
|                    SUBROUTINES
|
|
===============================================================*/

/*
	Using system coords, set global vars winX and winY which are OpenGL coords
	and mouseX and mouseY which are rotated coords
*/
void SetMouse(int x, int y)
{
	GLint viewport[4];
	GLdouble modelview[16];
	GLdouble projection[16];
	GLfloat winX, winY, winZ;
	

	glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
	glGetDoublev( GL_PROJECTION_MATRIX, projection );
	glGetIntegerv( GL_VIEWPORT, viewport );

	winX = (float)x;
	winY = (float)viewport[3] - (float)y;
	glReadPixels( x, int(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ );

	gluUnProject( winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);

	mouseX = translatefactor+(posX + posY)/(1.414213562373095048801);
	mouseY = -translatefactor+(posY - posX)/(1.414213562373095048801);


}

/*
		Create a DEBUG map
*/
void box(int row, int col)
{
		w[row][col].passable = 0;
		w[row][col].tiles[TILE_UP].index = TEX_WALL;
		w[row][col].tiles[TILE_UP].zoom = 0.7;
		w[row][col].tiles[TILE_DOWN].index = TEX_WALL;
		w[row][col].tiles[TILE_DOWN].zoom = 0.7;
		w[row][col].tiles[TILE_LEFT].index = TEX_WALL;
		w[row][col].tiles[TILE_LEFT].zoom = 0.7;
		w[row][col].tiles[TILE_RIGHT].index = TEX_WALL;
		w[row][col].tiles[TILE_RIGHT].zoom = 0.7;

		//w[row][col].tiles[TILE_ROOF].index = TEX_ROCKS;
		//w[row][col].tiles[TILE_ROOF].zoom = 0.7;
}

void fire(int row, int col, int index, int texture, int framec)
{
	objects[index].animation = texture;
		objects[index].framecount = framec;
		objects[index].framebegin = 0.0;
		objects[index].frameend = 1.0/(float)objects[index].framecount;
		objects[index].zoom = 15.0;
		objects[index].speed = 0.006;
		objects[index].worldCol = col;
		objects[index].worldRow = row;
		objects[index].realX = toScreen(objects[index].worldCol - viewportX)-0.1, 
		objects[index].realY =  -toScreen(objects[index].worldRow - viewportY);
		w[row][col].obj = &objects[index];
}



void initResources()
{
				int i;
	
		for(i=0;i<MAXTRIGGERS;i++)
			for(int j=0;j<MAXTRIGGERSOURCES;j++)
			{
				triggers[i].sources[j]=TRIGGER_NOBODY;
				triggers[i].targets[j]=TRIGGER_NOBODY;

				triggers[i].cause = TRIGGER_NONE;
			}

		int bla, bla2;
		
		for(i=0;i<WORLD_SIZE;i++)
		for(int j=0;j<WORLD_SIZE;j++){
			w[i][j].tiles[0].index = TEX_GRASS;
			w[i][j].tiles[1].index = TEX_CLEAR;
			w[i][j].tiles[2].index = TEX_CLEAR;
			w[i][j].tiles[3].index = TEX_CLEAR;
			w[i][j].tiles[4].index = TEX_CLEAR;
			w[i][j].tiles[5].index = TEX_CLEAR;
			w[i][j].tiles[6].index = TEX_CLEAR;
			w[i][j].tiles[0].zoom = 0.0;
			w[i][j].tiles[1].zoom = 0.0;
			w[i][j].tiles[2].zoom = 0.0;
			w[i][j].tiles[3].zoom = 0.0;
			w[i][j].tiles[4].zoom = 0.0;
			w[i][j].tiles[5].zoom = 0.0;
			w[i][j].tiles[6].zoom = 0.0;

			w[i][j].passable = 1;

	// VERY IMPORTANT TO NULL OUT
			w[i][j].charact = NULL;
			w[i][j].obj = NULL;
		}
	// VERY IMPORTANT TO NULL OUT
		for(i=0;i<MAXEVENTS;i++)events[i]=NULL;


	// VERY IMPORTANT TO NULL OUT
		for(i=0;i<MAXOBJECTS;i++)
		{
			objects[i].worldCol=-1;
			objects[i].worldRow=-1;
		}

	// VERY IMPORTANT TO NULL OUT
		for(i=0;i<MAXCHARACTERS;i++)
		{
			characters[i].worldCol=-1;
			characters[i].worldRow=-1;
		}



		//save
		/*FILE *fp;
		fp = fopen("Levels/default.tri", "wb");
		fwrite(&triggers[0], sizeof(struct Trigger), MAXTRIGGERS, fp);
		fclose(fp);*/

		//load map from default map file
		FILE *fp;
		
		if(!(fp = fopen("mapedit/in.map", "rb")))
		{
			MessageBox(NULL,"FATAL: Could not load map.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			exit(0);
		}
		fread(&w[0], sizeof(struct cell), WORLD_SIZE*WORLD_SIZE, fp);
		fclose(fp);

		if(!(fp = fopen("mapedit/in.obj", "rb")))
		{
			MessageBox(NULL,"FATAL: Could not load objects.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			exit(0);
		}
		fread(&objects[0], sizeof(struct object), MAXOBJECTS, fp);
		fclose(fp);



		// IMPORTANT !!!! create pointers from tiles to objects on them
		for(int objcnt=0;objcnt<MAXOBJECTS;objcnt++)
		{
			//this object is valid, not some junk
			if(objects[objcnt].worldCol>=0)
				w[objects[objcnt].worldRow][objects[objcnt].worldCol].obj = &objects[objcnt];
		}

		
}

void save_map()
{
	FILE *fp;
		
	//save map  WITH  references to objects already set
		if(!(fp = fopen("mapedit/default.map", "wb")))
		{
			MessageBox(NULL,"FATAL: Could not save map.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			exit(0);
		}
		fwrite(&w[0], sizeof(struct cell), WORLD_SIZE*WORLD_SIZE, fp);
		fclose(fp);


	//save objects
		if(!(fp = fopen("mapedit/default.obj", "wb")))
		{
			MessageBox(NULL,"FATAL: Could not save objects.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			exit(0);
		}
		fwrite(&objects[0], sizeof(struct object), MAXOBJECTS, fp);
		fclose(fp);
}


/*==============================================================
|
|
|                    DISPLAY SUBROUTINES
|
|
===============================================================*/



/*
	Draw all tiles in a cell using a <SMART> order
	indices:
		0 = floor
		1 = facing up
		2 = facing right
		3 = facing left
		4 = facing down
*/
void drawcell(double x, double y, struct cell c)
{
	//up
	glBindTexture(GL_TEXTURE_2D, tex[ c.tiles[1].index ].texID);
	drawtile(x, y, DRAW_TILE_UP, c.tiles[1].zoom, 0.0, 1.0);

	//right
	glBindTexture(GL_TEXTURE_2D, tex[ c.tiles[2].index ].texID);
	drawtile(x, y, DRAW_TILE_RIGHT, c.tiles[2].zoom, 0.0, 1.0);

	/*
	HERO/OBJECTS MUST BE DISPLAYED in drawobject()
	because lower tiles will overlap houses, etc.
	*/

	//NOT SURE IF I SHOULD LEAVE THIS HERE
	glBindTexture(GL_TEXTURE_2D, tex[ c.tiles[6].index ].texID);
	drawtile(x, y, DRAW_TILE_FLAT, c.tiles[6].zoom, 0.0, 1.0);

	drawobject(x, y, c);
	//drawcharacter(x, y, c);
	
	//left
	glBindTexture(GL_TEXTURE_2D, tex[ c.tiles[3].index ].texID);
	drawtile(x, y, DRAW_TILE_LEFT, c.tiles[3].zoom, 0.0, 1.0);

	//down
	glBindTexture(GL_TEXTURE_2D, tex[ c.tiles[4].index ].texID);
	drawtile(x, y, DRAW_TILE_DOWN, c.tiles[4].zoom, 0.0, 1.0);

}

void drawobject(double x, double y, struct cell c)
{
	if(c.obj!=NULL)
	{
		glBindTexture(GL_TEXTURE_2D, tex[ c.obj->animation ].texID);
		drawtile(c.obj->realX,
				c.obj->realY,
				DRAW_TILE_FLAT, 
				c.obj->zoom,
				c.obj->framebegin,
				c.obj->frameend);
	}
}

void drawcharacter(double x, double y, struct cell c){
	if(c.charact!=NULL)
	{onscreen++;


		//glScalef(1.0, 1.0, 1.0);

		glBindTexture(GL_TEXTURE_2D, tex[ c.charact->animation[c.charact->lastanimation]  ].texID);
		drawtile(c.charact->realX,
			     c.charact->realY,
				 //x,
				 //y,
				 DRAW_TILE_FLAT, 
				 c.charact->zoom,
				 c.charact->framebegin,
				 c.charact->frameend);
		//glScalef(0.75, 0.5, 1.0);
	}
}

/*
	Simply display a poly where the mouse is
*/
void drawmouse(double x, double y)
{
	glBindTexture(GL_TEXTURE_2D, mouse.texID);			// ( CHANGE )
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(x, y, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(x+0.05,  y, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(x+0.05, y-0.05, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(x, y-0.05, 0.0f);
	glEnd();
}



/*
	Draw one tile (it can be either facing down, up, floor, flat rendered)
	leftTex=0.0 and rightTex=1.0 unless drawing animations
*/
void drawtile(double x, double y, int position, double zoom, double leftTex, double rightTex)
{
	double factor = zoom * TILE_WIDTH;

	if(position==DRAW_TILE_UP)
	{
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(x+factor, 
											TILE_HEIGHT+y+factor, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(TILE_WIDTH+x+factor,
											TILE_HEIGHT+y+factor, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(x+TILE_WIDTH,
											TILE_HEIGHT+y, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(x,
										TILE_HEIGHT+y, 0.0f);
		glEnd();
	}

	if(position==DRAW_TILE_LEFT)
	{
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(x, 
											TILE_HEIGHT+y, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(factor+x,
											TILE_HEIGHT+y+factor, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(factor+x,
											y+factor, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(x,
										y, 0.0f);
		glEnd();
	}

	if(position==DRAW_TILE_RIGHT)
	{
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(1.0, 1.0f); glVertex3f(TILE_WIDTH+x+factor,
										y+factor, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(TILE_WIDTH+x+factor, 
											TILE_HEIGHT+y+factor, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(TILE_WIDTH+x,
											TILE_HEIGHT+y, 0.0f);	
		glTexCoord2f(1.0, 0.0f); glVertex3f(TILE_WIDTH+x,
											y, 0.0f);
		glEnd();
	}

	if(position==DRAW_TILE_DOWN)
	{
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(x+factor, 
											factor+y, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(TILE_WIDTH+x+factor,
											factor+y, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(TILE_WIDTH+x,
											y, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(x,
										y, 0.0f);
		glEnd();
	}

	if(position==DRAW_TILE_HORIZONTAL)
	{
		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 1.0f); glVertex3f(x+factor, 
											TILE_HEIGHT+y+factor, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(TILE_WIDTH+x+factor,
											TILE_HEIGHT+y+factor, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(x+TILE_WIDTH+factor,
											y+factor, 0.0f);
		glTexCoord2f(0.0, 0.0f); glVertex3f(x+factor,
										y+factor, 0.0f);
		glEnd();
	}

	/*if(position==DRAW_TILE_FLAT)
	{
		double tilt = TILE_HEIGHT/4.0;
		double t =sqrt(2.0)*TILE_WIDTH*zoom;

		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(leftTex, 1.0f); glVertex3f(tilt+x+TILE_WIDTH/2.0, 
											tilt+TILE_HEIGHT+y+TILE_HEIGHT/2.0*t, 0.0f);
		glTexCoord2f(rightTex, 1.0f); glVertex3f(tilt+TILE_WIDTH+x+TILE_WIDTH/2.0*t,
											tilt+TILE_HEIGHT+y-TILE_HEIGHT/2.0, 0.0f);
		glTexCoord2f(rightTex, 0.0f); glVertex3f(tilt+TILE_WIDTH+x-TILE_WIDTH/2.0,
											tilt+y-TILE_HEIGHT/2.0*t, 0.0f);
		glTexCoord2f(leftTex, 0.0f); glVertex3f(tilt+x-TILE_WIDTH/2.0*t,
										tilt+y+TILE_HEIGHT/2.0, 0.0f);
		glEnd();
	}*/

	float flatfactor = zoom * TILE_WIDTH / 25.0;

	if(position==DRAW_TILE_FLAT)
	{
		double tilt = TILE_HEIGHT/4.0;
		double t =sqrt(2.0)*TILE_WIDTH*zoom;

		glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(leftTex, 1.0f); glVertex3f(tilt+x+TILE_WIDTH/2.0 + flatfactor, 
											tilt+TILE_HEIGHT+y+TILE_HEIGHT/2.0*t + flatfactor, 0.0f);
		glTexCoord2f(rightTex, 1.0f); glVertex3f(tilt+TILE_WIDTH+x+TILE_WIDTH/2.0*t + flatfactor,
											tilt+TILE_HEIGHT+y-TILE_HEIGHT/2.0 + flatfactor, 0.0f);
		glTexCoord2f(rightTex, 0.0f); glVertex3f(tilt+TILE_WIDTH+x-TILE_WIDTH/2.0,
											tilt+y-TILE_HEIGHT/2.0*t, 0.0f);
		glTexCoord2f(leftTex, 0.0f); glVertex3f(tilt+x-TILE_WIDTH/2.0*t,
										tilt+y+TILE_HEIGHT/2.0, 0.0f);
		glEnd();
	}

	

}


void drawcorner()
{
	glColor3f(0.7, 0.7, 0.7);
	glBindTexture(GL_TEXTURE_2D, corner.texID);
	glBegin(GL_TRIANGLES);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(-1.78f, 0.33f, 0.0f);
		glTexCoord2f(0.0, 0.7f); glVertex3f(-1.78f, 1.60f, 0.0f);
		glTexCoord2f(1.0, 0.7f); glVertex3f(-0.52f, 1.60f, 0.0f);
	glEnd();

	glBegin(GL_TRIANGLES);										// Draw Our Quad
		glTexCoord2f(0.8, 0.7f); glVertex3f(1.78f, 1.60f, 0.0f);
		glTexCoord2f(0.8, 0.0f); glVertex3f(1.78f, 0.33f, 0.0f);
		glTexCoord2f(0.0, 0.7f); glVertex3f(0.52f, 1.60f, 0.0f);
	glEnd();

	glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(-0.59f, 1.55f, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(-0.59f, 1.60f, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(0.58f, 1.60f, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(0.58f, 1.55f, 0.0f);
	glEnd();

	glColor3f(0.7, 0.4, 0.4);
	float step = 0.4;
	//up and down
	for(float i=-1.78;i<=1.85-step;i+=step)
	{
	glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(i, -2.39f, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(i+step, -2.39f, 0.0f);
		glTexCoord2f(1.0, 1.0f); glVertex3f(i+step, -1.90f, 0.0f);
		glTexCoord2f(1.0, 0.0f); glVertex3f(i, -1.90f, 0.0f);
	glEnd();

	glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(i, 1.55f, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(i+step, 1.55f, 0.0f);
		glTexCoord2f(0.8, 1.0f); glVertex3f(i+step, 1.65f, 0.0f);
		glTexCoord2f(0.8, 0.0f); glVertex3f(i, 1.65f, 0.0f);
	glEnd();
	}


	glColor3f(0.7, 0.7, 0.7);
	glBegin(GL_TRIANGLES);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(-1.78f, -2.12f, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(-1.78f, -0.33f, 0.0f);
		glTexCoord2f(0.9, 0.0f); glVertex3f(0.0f, -2.12f, 0.0f);
	glEnd();

	glBegin(GL_TRIANGLES);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(0.0f, -2.12f, 0.0f);
		glTexCoord2f(0.8, 1.0f); glVertex3f(1.78f, -0.33f, 0.0f);
		glTexCoord2f(0.8, 0.0f); glVertex3f(1.78f, -2.12f, 0.0f);
	glEnd();


	//left and right
	glColor3f(0.7, 0.4, 0.4);
	for(i=-2.39;i<=1.70-step;i+=step)
	{
	glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(-1.78, i, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(-1.78, i+step, 0.0f);
		glTexCoord2f(0.7, 1.0f); glVertex3f(-1.70, i+step, 0.0f);
		glTexCoord2f(0.7, 0.0f); glVertex3f(-1.70, i, 0.0f);
	glEnd();
	glBegin(GL_QUADS);										// Draw Our Quad
		glTexCoord2f(0.0, 0.0f); glVertex3f(1.70, i, 0.0f);
		glTexCoord2f(0.0, 1.0f); glVertex3f(1.70, i+step, 0.0f);
		glTexCoord2f(0.7, 1.0f); glVertex3f(1.78, i+step, 0.0f);
		glTexCoord2f(0.7, 0.0f); glVertex3f(1.78, i, 0.0f);
	glEnd();
	}


	glColor3f(1.0, 1.0, 1.0);

}


void drawmap(){
	onscreen=0;
	int i,j;
	/*
	   First display all the floor tiles, so that they won't overlap
	   houses that are above them, but bigger
	*/
	for(i=0;i<=29;i++)
		for(j=0;j<=i;j++)
		{
			int col = 30-1-i+j;
			int row = j;

			glBindTexture(GL_TEXTURE_2D, tex[ w[row + viewportY][col + viewportX].tiles[TILE_FLOOR].index ].texID);
			drawtile(toScreen(col)-0.1, 
					 -toScreen(row),
					 DRAW_TILE_HORIZONTAL,
					 w[row + viewportY][col + viewportX].tiles[TILE_FLOOR].zoom, 
					 0.0, 1.0);
		}


	for(i=1;i<=29;i++)
		for(j=0;j<=30-i-1;j++)
		{
			int col = j;
			int row = j+i;
			glBindTexture(GL_TEXTURE_2D, tex[ w[row + viewportY][col + viewportX].tiles[TILE_FLOOR].index ].texID);
			drawtile(toScreen(col)-0.1, 
					 -toScreen(row),
					 DRAW_TILE_HORIZONTAL,
					 w[row + viewportY][col + viewportX].tiles[TILE_FLOOR].zoom, 
					 0.0, 1.0);
		}

	//=========DONE DRAWING FLOORS=================

	//=====================DRAW MAP TILES========================

	for(i=0;i<=29;i++)
		for(j=0;j<=i;j++)
		{
			int col = 30-1-i+j;
			int row = j;

			drawcell(toScreen(col)-0.1, -toScreen(row), w[row + viewportY][col + viewportX]);
		}

	for(i=1;i<=29;i++)
		for(j=0;j<=30-i-1;j++)
		{
			int col = j;
			int row = j+i;

			drawcell(toScreen(col)-0.1, -toScreen(row), w[row + viewportY][col + viewportX]);
		}
	//===========================================================


	/*
	   display all roof tiles
	*/
	for(i=0;i<=29;i++)
		for(j=0;j<=i;j++)
		{
			int col = 30-1-i+j;
			int row = j;

			glBindTexture(GL_TEXTURE_2D, tex[ w[row + viewportY][col + viewportX].tiles[TILE_ROOF].index ].texID);
			drawtile(toScreen(col)-0.1, 
					 -toScreen(row),
					 DRAW_TILE_HORIZONTAL,
					 w[row + viewportY][col + viewportX].tiles[TILE_ROOF].zoom, 
					 0.0, 1.0);
		}


	for(i=1;i<=29;i++)
		for(j=0;j<=30-i-1;j++)
		{
			int col = j;
			int row = j+i;
			glBindTexture(GL_TEXTURE_2D, tex[ w[row + viewportY][col + viewportX].tiles[TILE_ROOF].index ].texID);
			drawtile(toScreen(col)-0.1, 
					 -toScreen(row),
					 DRAW_TILE_HORIZONTAL,
					 w[row + viewportY][col + viewportX].tiles[TILE_ROOF].zoom, 
					 0.0, 1.0);
		}

	//=========DONE DRAWING ROOFS=================

		//COLOR CURSOR TILE

		int whichtile;

		if(currentori==TILE_FLOOR)whichtile=DRAW_TILE_HORIZONTAL;
		if(currentori==TILE_ROOF)whichtile=DRAW_TILE_HORIZONTAL;
		if(currentori==TILE_UP)whichtile=DRAW_TILE_UP;
		if(currentori==TILE_DOWN)whichtile=DRAW_TILE_DOWN;
		if(currentori==TILE_LEFT)whichtile=DRAW_TILE_LEFT;
		if(currentori==TILE_RIGHT)whichtile=DRAW_TILE_RIGHT;
		if(currentori==6)whichtile=DRAW_TILE_FLAT;

		glColor3f(1.0, 0.0, 1.0);
		glBindTexture(GL_TEXTURE_2D, tex[ w[mouseRow][mouseCol].tiles[currentori].index ].texID);
			drawtile(toScreen(mouseCol-viewportX)-0.1, 
					 -toScreen(mouseRow-viewportY),
					 whichtile,
					 w[mouseRow][mouseCol].tiles[currentori].zoom, 
					 0.0, 1.0);

		glColor3f(1.0, 1.0, 1.0);

		glBindTexture(GL_TEXTURE_2D, tex[TEX_WHITE_CONTOUR_FOR_MAPEDITOR_CURSOR].texID);
			drawtile(toScreen(mouseCol-viewportX)-0.1, 
					 -toScreen(mouseRow-viewportY),
					 DRAW_TILE_HORIZONTAL,
					 0.0, 
					 0.0, 1.0);
}




/*
	given a 'desired' maxdistance that character number charno wants to move in
	<direction> direction, find the maximum allowed distance it can move,
	stopping at walls, characters, objects, etc.
*/
double movement_getfarthest(int direction, double maxdistance, int charno)
{
	double distance = 0.0;
	int cCol = characters[charno].worldCol;
	int cRow = characters[charno].worldRow;

	if(direction==EVENT_UP)cRow--;
	if(direction==EVENT_DOWN)cRow++;
	if(direction==EVENT_LEFT)cCol--;
	if(direction==EVENT_RIGHT)cCol++;
	if(direction==EVENT_UPRIGHT){cRow--;cCol++;}
	if(direction==EVENT_UPLEFT){cRow--;cCol--;}
	if(direction==EVENT_DOWNRIGHT){cRow++;cCol++;}
	if(direction==EVENT_DOWNLEFT){cRow++;cCol--;}

	while(distance<maxdistance &&
		  w[cRow][cCol].passable &&
		  w[cRow][cCol].charact == NULL &&
		  w[cRow][cCol].obj == NULL)
	{
		distance += TILE_WIDTH;
		//now update coords
		if(direction==EVENT_UP)cRow--;
		if(direction==EVENT_DOWN)cRow++;
		if(direction==EVENT_LEFT)cCol--;
		if(direction==EVENT_RIGHT)cCol++;
		if(direction==EVENT_UPRIGHT){cRow--;cCol++;}
		if(direction==EVENT_UPLEFT){cRow--;cCol--;}
		if(direction==EVENT_DOWNRIGHT){cRow++;cCol++;}
		if(direction==EVENT_DOWNLEFT){cRow++;cCol--;}
	}

	//take the minimum of distance and maxdistance
	if(distance<=maxdistance)
		return distance;
	else
		return maxdistance;
	
}

/*
	Check if it's necessary to scroll viewport, and do it if yes
*/
void scroll_viewport()
{
	int scrolled = 0;

	//if char is within threshold units of edge, scroll
	int threshold = 5;
	//scroll by this many units if needed
	int amount = 5;

	//do we need to scroll right ?
	if(characters[0].worldCol >= (viewportX + 30) - threshold)
	{
		viewportX += amount;
		scrolled=1;
	}
	//do we need to scroll left ?
	else if(characters[0].worldCol <= (viewportX + threshold - 1))
	{
		viewportX -= amount;
		scrolled=1;
	}
	//do we need to scroll up ?
	else if(characters[0].worldRow <= (viewportY + threshold - 1))
	{
		viewportY -= amount;
		scrolled=1;
	}
	//do we need to scroll down ?
	else if(characters[0].worldRow >= (viewportY + 30)-threshold)
	{
		viewportY += amount;
		scrolled=1;
	}

	if(scrolled)
	{
		//recalculate all realX and realY for ALL other chars that are onscreen
		// ALSO recalculate for objects!!
		for(int col = 0;col<30;col++)
			for(int row = 0;row<30;row++)
			{
				struct cell current = w[row + viewportY][col + viewportX];

				if(w[row + viewportY][col + viewportX].charact != NULL)
				{
					current.charact->realX =  toScreen(current.charact->worldCol - viewportX)-0.1, 
					current.charact->realY = -toScreen(current.charact->worldRow - viewportY);
				}

				if(w[row + viewportY][col + viewportX].obj != NULL)
				{
					current.obj->realX =  toScreen(current.obj->worldCol - viewportX)-0.1, 
					current.obj->realY = -toScreen(current.obj->worldRow - viewportY);
				}
			}

		//recalculate screen coords for main guy
		characters[0].realX = toScreen(characters[0].worldCol - viewportX)-0.1, 
		characters[0].realY =  -toScreen(characters[0].worldRow - viewportY);
		
		//stop all char movement and static animations
		for(int count = 0;count<MAXEVENTS;count++)
		{
			if(events[count]!=NULL && events[count]->type==EVENT_MOVEMENT)
			{
				delete events[count];
				events[count]=NULL;
			}
			if(events[count]!=NULL && events[count]->type==EVENT_STATIC)
			{
				delete events[count];
				events[count]=NULL;
			}
		}
	}
}

/*
	try to move character <charnum>
*/
int attempt_movement(int charnum, int direction, float maxdistance)
{
	//dont even bother if he's dead
	if(characters[charnum].life<=0 || characters[charnum].alive==0)
		return 0;

	int count;

	//first check if characters[charnum] is doing anything right now
	for(count = 0;count<MAXEVENTS;count++)
	{
		if(events[count]!=NULL)
		{
			//is it moving or dying ?
			if(events[count]->charact==&characters[charnum]) break;

			//is it attacking anyone else ?
			else if(events[count]->type==EVENT_MELEE &&
				    events[count]->attacker==&characters[charnum]) break;
			//is it being hit ?
			else if(events[count]->type==EVENT_HIT &&
					events[count]->victim==&characters[charnum]) break;
		}
	}

	//we didn't reach end of events array, so we found an event that involves charnum
	//so we need to return immediately
	if(count!=MAXEVENTS) return 0;

	//see what is the farthest we can move until we reach an unpassable cell
	double farthest = 0.0;
	farthest = movement_getfarthest(direction, maxdistance, charnum);

	//we can move
	if(farthest>0.0)
	{
		//for our main guy, we always have the event at index 0
		if(charnum==0)
		{
			events[0]=new Event(&w[0][0], &characters[0], EVENT_MOVEMENT, direction, farthest, farthest);
			events[0]->startmovement();
			events[0]->setviewport(viewportX, viewportY);
		}
		//now handle any other character (NPC)
		if(charnum!=0)
		{
			//find an empty spot in the event array
			int empty_spot = 1;
			while(empty_spot<MAXEVENTS)
			{
				if(events[empty_spot]==NULL)break;
				empty_spot++;
			}

			//we didn't reach end of events array, so we found a spot
			if(empty_spot<MAXEVENTS)
			{
				events[empty_spot]=new Event(&w[0][0], &characters[charnum], EVENT_MOVEMENT, direction, farthest, farthest);
				events[empty_spot]->startmovement();
				events[empty_spot]->setviewport(viewportX, viewportY);
			}
		}
	}

	return 1;
}


/*
	Make all onscreen NPCs follow character

	NOTE: NPC will always attempt twice or more to move, in order to be able to
		  try different routes in case he's blocked by a wall or something else.
*/
void npc_follow()
{
	for(int col = 0;col<30;col++)
			for(int row = 0;row<30;row++)
			{
				struct character *current = w[row + viewportY][col + viewportX].charact;
				//if there's a NPC (other than main guy) on this cell
				if(current != NULL &&
				   current != &characters[0])
				{
					//find index of this char inside characters array
					int char_index=0;
					while(current != &characters[char_index]  &&
							char_index<MAXCHARACTERS)  //although if the char is on a cell, it will ALWAYS be
													   //in characters array, it's just for extra-caution
					{
						char_index++;
					}

					//FIND A TARGET HERE !
					int target_index = npc_findtarget(current);
					if(target_index==-1)
					{
						//no target was found!! ignore this char then..
						continue;
					}

					int targetCol=characters[target_index].worldCol;
					int targetRow=characters[target_index].worldRow;

					

					int col_diff = current->worldCol - 
								   targetCol;
					int row_diff = current->worldRow - 
								   targetRow;

					//=====WE ARE CLOSE ENOUGH TO ATTACK !===
					if(abs(col_diff)<=current->radius &&
					   abs(row_diff)<=current->radius)
					{
						attempt_melee_attack(char_index, target_index);
					}

					//target is left from NPC
					if(col_diff>0)
					{
						//directly left
						if(row_diff==0)
						{
							attempt_movement(char_index, EVENT_LEFT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_DOWNLEFT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_UPLEFT, 1*TILE_WIDTH);
						}

						//below
						if(row_diff<0)
						{
							attempt_movement(char_index, EVENT_DOWNLEFT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_LEFT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_DOWN, 1*TILE_WIDTH);
						}

						//above
						if(row_diff>0)
						{
							attempt_movement(char_index, EVENT_UPLEFT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_UP, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_LEFT, 1*TILE_WIDTH);
						}
					}//end if target is left from NPC

					//mtarget is right from NPC
					if(col_diff<0)
					{
						//directly right
						if(row_diff==0)
						{
							attempt_movement(char_index, EVENT_RIGHT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_DOWNRIGHT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_UPRIGHT, 1*TILE_WIDTH);
						}

						//below
						if(row_diff<0)
						{
							attempt_movement(char_index, EVENT_DOWNRIGHT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_DOWN, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_RIGHT, 1*TILE_WIDTH);
						}

						//above
						if(row_diff>0)
						{
							attempt_movement(char_index, EVENT_UPRIGHT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_RIGHT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_UP, 1*TILE_WIDTH);
						}
					}//end if target is right from NPC

					//target is directly above or below NPC
					if(col_diff==0)
					{
						//above
						if(row_diff>0)
						{
							attempt_movement(char_index, EVENT_UP, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_UPLEFT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_UPRIGHT, 1*TILE_WIDTH);
						}
						//below
						if(row_diff<0)
						{
							attempt_movement(char_index, EVENT_DOWN, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_DOWNRIGHT, 1*TILE_WIDTH);
							attempt_movement(char_index, EVENT_DOWNLEFT, 1*TILE_WIDTH);
						}
					}


				}
			}
}



/*
	For this charact, try to find a suitable target within its aggro radius
	based on their alignment

	return: index of target in the characters array
*/
int npc_findtarget(struct character *charact)
{
	//search in widening squares
	for(int i=1;i<=charact->aggro;i++)
	{
		int minCol = max( viewportX, charact->worldCol- i );
		int maxCol = min( viewportX+29, charact->worldCol+ i );

		int minRow = max( viewportY, charact->worldRow- i );
		int maxRow = min( viewportY+29, charact->worldRow+ i );

		for(int row=minRow; row<=maxRow; row++)
			for(int col=minCol; col<=maxCol; col++)
			{
				struct character* target = w[row][col].charact;
				if( target!=NULL &&
					target->alive &&
					target->alignment * charact->alignment < 0)
				{
					//ok, we found a target
					//find index of target inside characters array
					int char_index=0;
					while(target != &characters[char_index]  &&
							char_index<MAXCHARACTERS)  //although if the char is on a cell, it will ALWAYS be
													   //in characters array, it's just for extra-caution
					{
						char_index++;
					}
					//char_index contains the index of our target
					return char_index;
				}
			}
	}


	return -1;
}



/*
	att tries to melee attack vic
*/
int attempt_melee_attack(int att, int vic)
{
	//exit immediately if either attacker or victim is dead
	if(characters[att].life<=0 ||
	   characters[att].alive==0 ||
	   characters[vic].life<=0 ||
	   characters[vic].alive==0) return 0;
	//exit immediately if their alignments are not opposite
	if(characters[att].alignment * characters[vic].alignment != -1) return 0;
	int count;

	//first check if attacker is doing anything
	for(count = 0;count<MAXEVENTS;count++)
	{
		if(events[count]!=NULL)
		{
			//is it moving or dying ?
			if(events[count]->charact==&characters[att]) break;

			//is it attacking anyone else ?
			else if(events[count]->type==EVENT_MELEE &&
				    events[count]->attacker==&characters[att]) break;
			//is it being hit ?
			else if(events[count]->type==EVENT_HIT &&
					events[count]->victim==&characters[att]) break;
		}
	}

	//we didn't reach end of events array, so we found an event that involves charnum
	//so we need to return immediately
	if(count!=MAXEVENTS) return 0;

	//we're ok; we can attack
	//find an empty spot in the event array
			int empty_spot = 1;
			while(empty_spot<MAXEVENTS)
			{
				if(events[empty_spot]==NULL)break;
				empty_spot++;
			}

			//we didn't reach end of events array, so we found a spot
			if(empty_spot<MAXEVENTS)
			{
				events[empty_spot]=new Event(&characters[att], &characters[vic], EVENT_MELEE, TILE_WIDTH, 0);
				int damage = events[empty_spot]->startmelee();
				
				//did we land a hit ?
				if(damage!=0)
				{
					//create an EVENT_HIT if victim won't die from this attack
					if(characters[vic].life > 0)
					{
						int count;
	
						//first check if attacker is doing anything
						for(count = 0;count<MAXEVENTS;count++)
						{
							if(events[count]!=NULL)
							{
							//is it moving or dying?
							if(events[count]->charact==&characters[vic]) break;
				
							//is it attacking anyone else ?
							else if(events[count]->type==EVENT_MELEE &&
									events[count]->attacker==&characters[vic]) break;
							//is it being hit ?
							else if(events[count]->type==EVENT_HIT &&
									events[count]->victim==&characters[vic]) break;
							}
						}

					//we didn't reach end of events array, so we found an event that involves charnum
					//so we need to return immediately
					if(count!=MAXEVENTS) return 0;

					//we're ok, so show animation of target being hit
					//find an empty spot in the event array
					empty_spot = 1;
					while(empty_spot<MAXEVENTS)
					{
						if(events[empty_spot]==NULL)break;
						empty_spot++;
					}

					//we didn't reach end of events array, so we found a spot
					if(empty_spot<MAXEVENTS)
					{
						events[empty_spot]=new Event(&characters[att], &characters[vic], EVENT_HIT, TILE_WIDTH, damage);
						events[empty_spot]->starthit();
					}


					}//end if life of victim > 0 (survives)
					 //NOTE: if victim dies, it will be taken care of by grim_reaper()
				}//end if damage!=0  (the attack was successful)
			}
}

/*
	Make sure all object that are onscreen are animated
*/
int activate_objects()
{
	for(int col = 0;col<30;col++)
			for(int row = 0;row<30;row++)
			{
				struct object *current;
				current = w[row + viewportY][col + viewportX].obj;

				//if there's an object on this cell
				if(current != NULL)
				{
					//set it's real coords, since it's onscreen for sure
					current->realX =  toScreen(current->worldCol - viewportX)-0.1, 
					current->realY = -toScreen(current->worldRow - viewportY);

					int busy=0;
					//first check if obj is doing anything right now
					for(int count = 0;count<MAXEVENTS;count++)
					{
						if(events[count]!=NULL && 
						   events[count]->obj == current)
						{
							busy = 1;
							break;
						}
					}

					if(busy)continue;

					//so object is not doing anything right now....
					//find an empty spot in the event array
					int empty_spot = 1;
					while(empty_spot<MAXEVENTS)
					{
						if(events[empty_spot]==NULL)break;
						empty_spot++;
					}
		
					//we didn't reach end of events array, so we found a spot
					if(empty_spot<MAXEVENTS)
					{
						events[empty_spot]=new Event(&w[0][0], current, EVENT_STATIC, 0, TILE_WIDTH, TILE_HEIGHT);
						events[empty_spot]->startstatic();
					}
				}
			}

	return 1;
}


/*
	Check all cells on screen to see if there are characters with life <= 0 who have to die
*/
void grim_reaper()
{
	for(int col = 0;col<30;col++)
			for(int row = 0;row<30;row++)
			{
				struct character *current = w[row + viewportY][col + viewportX].charact;
				if(current!=NULL && 
				   (current->life<=0 || current->alive==0))
				{
					//this char has to die
					int count;

					current->life=0;
					current->alive=0;

					//first check if characters[charnum] is doing anything right now
					for(count = 0;count<MAXEVENTS;count++)
					{
						if(events[count]!=NULL)
						{
							//is it moving or dying ?
							if(events[count]->charact==current) break;
				
							//is it attacking anyone else ?
							else if(events[count]->type==EVENT_MELEE &&
									events[count]->attacker==current) break;
							//is it being hit ?
							else if(events[count]->type==EVENT_HIT &&
									events[count]->victim==current) break;
						}
					}
				
					//we didn't reach end of events array, so we found an event that involves current
					if(count!=MAXEVENTS) continue;

					//find an empty spot in the event array
					int empty_spot = 1;
					while(empty_spot<MAXEVENTS)
					{
						if(events[empty_spot]==NULL)break;
						empty_spot++;
					}

					//we didn't reach end of events array, so we found a spot
					if(empty_spot<MAXEVENTS)
					{
						events[empty_spot]=new Event(&w[row + viewportY][col + viewportX], 
							                         current, EVENT_DEATH, TILE_WIDTH);
						events[empty_spot]->startdeath();
					}

				}
			}
}



int mouse_handle()
{
	w[mouseRow][mouseCol].tiles[currentori].index = currenttex;

	return 1;
}


/*
=========================================
|
|       HANDLE
|				TRIGGERS
|
==========================================
*/

void check_triggers()
{
	for(int i=0;i<MAXTRIGGERS;i++)
	{
		check_trigger(&triggers[i]);
	}//end for all triggers
}

/*
	See if anything was triggered, according to conditions
*/
int check_trigger(struct Trigger *t)
{
		//skip to next trigger if this one does nothing
		if(t->cause==TRIGGER_NONE)return 0;

		//if this trigger is not on-screen, ignore it
		if(t->worldCol<viewportX || t->worldCol>viewportX+29 ||
		   t->worldRow<viewportY || t->worldRow>viewportY+29) return 0;

		//we'll have an if for each t->cause
		if(t->cause==TRIGGER_WALKOVER)
		{
			//valid source?
			if(t->sources[0]<0)return 0;

			//only works for first source, which cannot be TRIGGER_ALL
			//see if source is ON that cell
			if(t->worldCol==characters[t->sources[0]].worldCol &&
			   t->worldRow==characters[t->sources[0]].worldRow)
			{
				act_trigger(t);
			}
		}//endif t->cause==walkover


		//death of one or more characters
		if(t->cause==TRIGGER_DEATH)
		{
			//ALL sources dead ?
			int valid=1;
			int count=0;
			while(count<MAXTRIGGERSOURCES)
			{
				if(t->sources[count]!=TRIGGER_NOBODY && characters[t->sources[count]].alive)
								valid=0;
				count++;
			}
			if(valid) act_trigger(t);
		}

		//one or more characters are ALL alive
		if(t->cause==TRIGGER_ALIVE)
		{
			//ALL sources alive ?
			int valid=1;
			int count=0;
			while(count<MAXTRIGGERSOURCES)
			{
				if(t->sources[count]!=TRIGGER_NOBODY && !characters[t->sources[count]].alive)
								valid=0;
				count++;
			}
			if(valid) act_trigger(t);
		}

}

/*
	t must be triggered, so do it here, according to different effects
*/
int act_trigger(struct Trigger *t)
{
	if(t->effect==TRIGGER_NONE)return 0;

	//FIRST, see if Trigger acts on tiles rather than characters (TRIGGER_ACTION_OPEN)
	if(t->effect==TRIGGER_ACTION_OPEN)
	{
		//make it passable
		w[t->worldRow][t->worldCol].passable = 1;

		//NOTE: coords of tile to be OPENED are contained in Trigger.worldRow and worldCol
		//      also, the index of flat rendered
		//ANOTHER NOTE: indices of textures are in targets[]. This way we can OPEN floor, ceiling, etc.
		for(int index=0;index<MAXTRIGGERSOURCES;index++)
		{
			//if not a valid index, stop
			if(t->targets[index]<0)break;

			//it's valid, so change its texture
			w[t->worldRow][t->worldCol].tiles[t->targets[index]].index = t->direction;
		}

		//if we only have to do it once, clear this trigger forever
				if(t->longevity==TRIGGER_LONGEVITY_ONCE)
						t->cause = TRIGGER_NONE;

		//we don't want to treat this Trigger as involving chars, so just quit now
		return 1;
	}
		

	//for all targets (since it's not TRIGGER_ACTION_OPEN, so it acts on characters)
	for(int index=0;index<MAXTRIGGERSOURCES;index++)
	{
		//if not a valid index for target NPC, stop at once: we won't find any valid ones
		if(t->targets[index]<0)break;

		struct character *c = &characters[t->targets[index]];

		if(c->alive==0 || c->life<=0)break;
		
		//see if target character is on-screen
		if(c->worldCol>=viewportX && c->worldCol<=viewportX+29 &&
		   c->worldRow>=viewportY && c->worldRow<=viewportY+29)
		{
			//is the effect of this trigger a movement ?
			//The target MUST be onscreen for movement
			if(t->effect==TRIGGER_ACTION_MOVE)
			{
				attempt_movement(t->targets[index], t->direction, t->distance);
				//if we only have to do it once, clear this trigger forever
				if(t->longevity==TRIGGER_LONGEVITY_ONCE)
						t->cause = TRIGGER_NONE;
			}//end if effect is ACTION_MOVE



		}//end if target is onscreen

		//we have to kill this target
		if(t->effect==TRIGGER_ACTION_KILL)
		{
			c->alive = 0;
			c->life = -50;
			//clear trigger if we only do it once
			if(t->longevity==TRIGGER_LONGEVITY_ONCE)
						t->cause = TRIGGER_NONE;
		}

		//vanish a NPC from this world. do NOT kill him (no dying animation)
		if(t->effect==TRIGGER_ACTION_VANISH)
		{
			w[c->worldRow][c->worldCol].charact = NULL;
			//clear trigger if we only do it once
			if(t->longevity==TRIGGER_LONGEVITY_ONCE)
						t->cause = TRIGGER_NONE;
		}

		//change a NPC's alignment, using Trigger.direction as new alignment
		if(t->effect==TRIGGER_ACTION_CONVERT)
		{
			c->alignment = t->direction;
			//clear trigger if we only do it once
			if(t->longevity==TRIGGER_LONGEVITY_ONCE)
						t->cause = TRIGGER_NONE;
		}


	}//end for all targets

	return 1;
}


void add_object()
{
	//only add object IF mouse was on playarea AND there's no object there already
	if(mouseRow>-1 && mouseCol>-1 &&
	   w[mouseRow][mouseCol].obj==NULL)
	{

		lastobjectindex++;
		objects[lastobjectindex].animation = currenttex;
		
		objects[lastobjectindex].worldCol = mouseCol;
		objects[lastobjectindex].worldRow = mouseRow;
	
		objects[lastobjectindex].zoom = 7.0;
		objects[lastobjectindex].framecount = 4;
		objects[lastobjectindex].speed = 0.008;
	
		w[mouseRow][mouseCol].obj = &objects[lastobjectindex];
	}
}
